<?php

/**
 * @file
 * Handle the base functionality for querying the Pin API.
 */


/**
 * A simple wrapper class for our endpoints.
 */
class CommercePinEndpoints {
  const endpoint_live = 'api.pin.net.au/1';
  const endpoint_test = 'test-api.pin.net.au/1';
}


/**
 * The base class for a Pin transaction.
 */
abstract class CommercePinTransactionBase {

  private $secret_key = FALSE;

  /**
   * @param $secret_key
   *   The secret key used to query the API.
   * @param bool $test_mode
   *   If we are going to query the API in test mode.
   */
  function __construct($secret_key, $test_mode = FALSE) {
    $this->secret_key = $secret_key;
    $this->test_mode = $test_mode;
  }

  /**
   * @return String
   *   The endpoint for the transaction including the authentication details.
   */
  function getApiBase() {

    $url = $this->test_mode
      ? CommercePinEndpoints::endpoint_test
      : CommercePinEndpoints::endpoint_live;

    return 'https://' . $this->secret_key . ':@' . $url;
  }

  /**
   * Query the Pin API.
   *
   * @param $endpoint
   *   The endpoint we are querying.
   * @param $variables
   *   A list of variables to send to the gateway.
   * @return CommercePinResponse
   *   A response object containing information about the request.
   */
  function queryPinApi($endpoint, $variables) {

    // The endpoint URL we will query.
    $endpoint_url = $this->getApiBase() . '/' . $endpoint . '/';

    // Add some standard variables to our request variables.
    $request_variables = array(
      'ip_address' => $_SERVER['REMOTE_ADDR'],
    ) + $variables;

    $encoded_request_variables = json_encode($request_variables);

    // Get a curl request and set the correct endpoint.
    $request = $this->createCurlPostRequest();

    curl_setopt($request, CURLOPT_URL, $endpoint_url);
    curl_setopt($request, CURLOPT_POSTFIELDS, $encoded_request_variables);
    curl_setopt($request, CURLOPT_HTTPHEADER,
      array(
        'Content-Length: ' . strlen($encoded_request_variables),
        'Content-Type: application/json',
      )
    );

    // Get the response from Pin.
    $response = json_decode(curl_exec($request));

    // Return a newly constructed response object.
    return new CommercePinResponse($response);
  }

  /**
   * Create a curl POST request and return a curl handler.
   *
   * @return resource $ch
   *   A new curl request.
   */
  function createCurlPostRequest() {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    return $ch;
  }

}
